package com.hello2morrow.sonargraph.languageprovider.java.command.system;

import static org.junit.Assert.*

import org.junit.Test

import com.hello2morrow.sonargraph.api.model.Aggregator
import com.hello2morrow.sonargraph.core.api.controller.ICoreVisitor
import com.hello2morrow.sonargraph.core.api.model.SourceFileAccess
import com.hello2morrow.sonargraph.core.api.model.TypeAccess
import com.hello2morrow.sonargraph.core.api.script.model.NodeAccess
import com.hello2morrow.sonargraph.core.api.script.model.ResultAccess
import com.hello2morrow.sonargraph.core.controller.system.GroovyExtension
import com.hello2morrow.sonargraph.core.controller.system.SoftwareSystemController.OptionalExtensionId
import com.hello2morrow.sonargraph.core.controller.system.script.CoreAccess
import com.hello2morrow.sonargraph.core.model.element.ShortNameFilter
import com.hello2morrow.sonargraph.core.model.script.FullGroovyScript
import com.hello2morrow.sonargraph.core.model.script.GroovyScript
import com.hello2morrow.sonargraph.core.model.script.GroovyScripts
import com.hello2morrow.sonargraph.core.model.script.IScriptApi
import com.hello2morrow.sonargraph.core.model.system.Files
import com.hello2morrow.sonargraph.core.model.system.SoftwareSystem
import com.hello2morrow.sonargraph.core.model.system.SoftwareSystemState
import com.hello2morrow.sonargraph.core.model.system.ISoftwareSystemProvider.OpenOption
import com.hello2morrow.sonargraph.languageprovider.java.foundation.common.JavaTestFixture

import de.schlichtherle.truezip.file.TFile

public class ScriptApiTest extends AbstractJavaCommandTest
{
    private static final String PROJECT_PATH = JavaTestFixture.SMALL_TEST_PROJECT;
    private GroovyScript m_testScript;

    public ScriptApiTest()
    {
        super(PROJECT_PATH);
    }

    @Test
    public void testLinesOfCode()
    {
        Map<String, Object> roots = initSoftwareSystem();

        CoreAccess coreSystem = (CoreAccess) roots.get("coreAccess");
        ResultAccess result = (ResultAccess) roots.get("result");

        def totalLinesOfCode = 0;
        def totalSourceFiles = 0;
        def totalModules = 0;
        def NodeAccess over1000Loc = result.addNode("Over 1000");

        def sourceFileClosure =
        { SourceFileAccess sourceFile ->

            if(!sourceFile.isExternal() && !sourceFile.isExcluded())
            {
                result.addElement(sourceFile);
                totalSourceFiles++

                def nextLoc = sourceFile.getLinesOfCodeMetric()
                if(nextLoc>1000)
                {
                    result.addNode(over1000Loc, sourceFile)
                }
                totalLinesOfCode+= nextLoc
            }
        }

        def moduleClosure =
        {
            totalModules++;
            visitor.visitChildren(it)
        }

        ICoreVisitor v = coreSystem.createVisitor();
        v.onSourceFile(sourceFileClosure);
        v.onModule(moduleClosure);
        coreSystem.visitModel(v);

        assertTrue("Source files expected", totalSourceFiles > 0);
        assertTrue("Total lines of code must be > 140, but was " + totalLinesOfCode, totalLinesOfCode > 140);
        assertEquals("Number of modules does not match", 4, totalModules);
        assertEquals("No file with more than 1000 LOC expected", 0, over1000Loc.getNode().numberOfChildren);
    }

    @Test
    public void testFindClassesWithOneOrZeroIncomingDependency()
    {
        Map<String, Object> roots = initSoftwareSystem();

        CoreAccess coreSystem = (CoreAccess) roots.get("coreAccess");
        ResultAccess result = (ResultAccess) roots.get("result");

        def NodeAccess zeroOrOneIncomingDep = result.addNode("0 or 1 incoming dependency");

        Closure closure =
        { TypeAccess type ->

            if(!type.isExternal() && !type.isExcluded())
            {
                int numberOfIncomingDependencies = type.getIncomingDependenciesRecursively(Aggregator.TYPE, true).size();

                if(numberOfIncomingDependencies <= 1)
                {
                    NodeAccess node = result.addNode(zeroOrOneIncomingDep, type);

                    result.addWarningIssue(type, "0 or 1 internal dependency", "Type has 0 or 1 incoming dependency");
                }
            }
        }

        ICoreVisitor v = coreSystem.createVisitor()

        v.onType(closure)
        coreSystem.visitModel(v);

        assertEquals("Wrong number of classes with few incoming dependencies", 5, zeroOrOneIncomingDep.getNode().numberOfChildren);
        assertNotNull(zeroOrOneIncomingDep.getNode().children.find { it.name == "com.h2m.alarm.model.AlarmClock" });
        assertNotNull(zeroOrOneIncomingDep.getNode().children.find { it.name == "com.h2m.alarm.application.Main" });
    }

    private Map initSoftwareSystem()
    {
        openAndClearAndRefreshSystem(new TFile(PROJECT_PATH), EnumSet.noneOf(OpenOption.class), EnumSet.noneOf(OptionalExtensionId.class));
        SoftwareSystem softwareSystem = getController().getSoftwareSystem();
        assertEquals(SoftwareSystemState.MODEL_LOADED, softwareSystem.getState());

        GroovyScripts scripts = softwareSystem.getUniqueExistingChild(Files.class).getGroovyScripts();
        m_testScript = scripts.getUniqueChild(new ShortNameFilter("CompilationError.scr"), GroovyScript.class);
        assertNotNull("Script 'CompilationError.scr' expected", m_testScript);

        GroovyExtension groovyExtension = softwareSystem.getExtension(GroovyExtension.class);
        FullGroovyScript fullScript = groovyExtension.getFullScript(m_testScript, null, "", 0);
        IScriptApi api = fullScript.getScriptApi();

        Map<String, Object> roots = api.getRoots()
        return roots;
    }
}